@@ -1,10 +1,13 @@
 import re
 from enum import Enum
+from pathlib import Path
 from typing import Any, Dict, List, Literal, Optional, TypeVar, Union
+from urllib.parse import urlparse
 
-from pydantic import ConfigDict, Field
+from pydantic import ConfigDict, Field, ValidationError, field_validator
 from typing_extensions import Annotated, TypeAlias
 
+from mistral_common.audio import EXPECTED_FORMAT_VALUES, Audio
 from mistral_common.base import MistralBase
 from mistral_common.image import SerializableImage
 from mistral_common.protocol.instruct.tool_calls import ToolCall
@@ -17,6 +20,8 @@ class ChunkTypes(str, Enum):
        text: A text chunk.
        image: An image chunk.
        image_url: An image url chunk.
+       input_audio: An input audio chunk.
+       audio_url: An audio url chunk.
 
     Examples:
         >>> from mistral_common.protocol.instruct.messages import ChunkTypes
@@ -26,6 +31,9 @@ class ChunkTypes(str, Enum):
     text = "text"
     image = "image"
     image_url = "image_url"
+    input_audio = "input_audio"
+    audio_url = "audio_url"
+    thinking = "thinking"
 
 
 class BaseContentChunk(MistralBase):
@@ -37,7 +45,14 @@ class BaseContentChunk(MistralBase):
        type: The type of the chunk.
     """
 
-    type: Literal[ChunkTypes.text, ChunkTypes.image, ChunkTypes.image_url]
+    type: Literal[
+        ChunkTypes.text,
+        ChunkTypes.image,
+        ChunkTypes.image_url,
+        ChunkTypes.input_audio,
+        ChunkTypes.audio_url,
+        ChunkTypes.thinking,
+    ]
 
     def to_openai(self) -> Dict[str, Union[str, Dict[str, str]]]:
         r"""Converts the chunk to the OpenAI format.
@@ -142,6 +157,195 @@ class ImageURLChunk(BaseContentChunk):
         return cls.model_validate({"image_url": openai_chunk["image_url"]})
 
 
+class RawAudio(MistralBase):
+    r"""Base64 encoded audio data.
+
+    This class represents raw audio data encoded in base64 format.
+
+    Attributes:
+        data: The base64 encoded audio data, which can be a string or bytes.
+        format: The format of the audio data.
+
+    Examples:
+        >>> audio = RawAudio(data="base64_encoded_audio_data", format="mp3")
+    """
+
+    data: Union[str, bytes]
+    format: str
+
+    @classmethod
+    def from_audio(cls, audio: Audio) -> "RawAudio":
+        """Creates a RawAudio instance from an Audio object.
+
+        Args:
+            audio: An Audio object containing audio data, format, and duration.
+
+        Returns:
+            An AudioChunk instance initialized with the audio data.
+        """
+        format = audio.format
+        data = audio.to_base64(format, False)
+
+        return cls(data=data, format=format)
+
+    @field_validator("format")
+    def should_not_be_empty(cls, v: str) -> str:
+        if v not in EXPECTED_FORMAT_VALUES:
+            raise ValidationError(f"`format` should be one of {EXPECTED_FORMAT_VALUES}. Got: {v}`")
+
+        return v
+
+
+class AudioURL(MistralBase):
+    r"""Audio URL.
+
+    Attributes:
+        url: The URL of the audio file.
+    """
+
+    url: str
+
+
+class AudioURLType(str, Enum):
+    r"""Enum for the types of audio URLs.
+
+    Attributes:
+        url: A URL.
+        base64: A base64 encoded audio. Can be prefixed with `data:audio/<format>;base64,`.
+        file: A file path.
+        file_uri: A file URI (eg. `file:///path/to/file`).
+    """
+
+    url = "url"
+    base64 = "base64"
+    file = "file"
+    file_uri = "file_uri"
+
+
+class AudioURLChunk(BaseContentChunk):
+    r"""Audio URL chunk.
+
+    Attributes:
+        type: The type of the chunk, which is always `ChunkTypes.audio_url`.
+        audio_url: The URL of the audio file.
+    """
+
+    type: Literal[ChunkTypes.audio_url] = ChunkTypes.audio_url
+    audio_url: Union[str, AudioURL]
+
+    @property
+    def url(self) -> str:
+        if isinstance(self.audio_url, AudioURL):
+            return self.audio_url.url
+        return self.audio_url
+
+    def get_url_type(self) -> AudioURLType:
+        r"""Returns the type of the audio URL.
+
+        Note:
+            URLs should be either:
+            - a valid URL (http:// or https://)
+            - a valid file path (e.g. /path/to/file)
+            - a valid file URI (e.g. file:///path/to/file)
+            - a base64 encoded audio. It is assumed to be base64 encoded if it is not a valid URL or file path.
+
+        Returns:
+            The type of the audio URL.
+        """
+        url_scheme = urlparse(self.url).scheme
+        if url_scheme in {"http", "https"}:
+            return AudioURLType.url
+        elif url_scheme == "data":
+            return AudioURLType.base64
+        elif url_scheme == "file":
+            return AudioURLType.file_uri
+
+        try:
+            url_path = Path(self.url)
+            exist_path = url_path.exists()
+        except OSError:  # File name too long
+            exist_path = False
+
+        if exist_path:
+            return AudioURLType.file
+
+        return AudioURLType.base64
+
+    def to_openai(self) -> Dict[str, Union[str, Dict[str, str]]]:
+        r"""Converts the chunk to the OpenAI format."""
+        if isinstance(self.audio_url, AudioURL):
+            return self.model_dump()
+        else:
+            return {"type": self.type, "audio_url": {"url": self.audio_url}}
+
+    @classmethod
+    def from_openai(cls, openai_chunk: Dict[str, Union[str, Dict[str, str]]]) -> "AudioURLChunk":
+        r"""Converts the OpenAI chunk to the Mistral format."""
+        return cls.model_validate(openai_chunk)
+
+
+class AudioChunk(BaseContentChunk):
+    r"""Audio chunk containing raw audio data.
+
+    This class represents a chunk of audio data that can be used as input.
+
+    Attributes:
+        type: The type of the chunk, which is always ChunkTypes.input_audio.
+        input_audio: The RawAudio object containing the audio data.
+
+    Examples:
+        >>> audio_chunk = AudioChunk(input_audio=RawAudio(data="base64_encoded_audio_data", format="mp3"))
+    """
+
+    type: Literal[ChunkTypes.input_audio] = ChunkTypes.input_audio
+    input_audio: RawAudio
+
+    @field_validator("input_audio")
+    def should_not_be_empty(cls, v: RawAudio) -> RawAudio:
+        if not v.data.strip():
+            raise ValidationError(f"`InputAudio` should not be empty. Got: {v}`")
+
+        return v
+
+    @classmethod
+    def from_audio(cls, audio: Audio) -> "AudioChunk":
+        r"""Creates an AudioChunk instance from an Audio object.
+
+        Args:
+            audio: An Audio object containing audio data.
+
+        Returns:
+            An AudioChunk instance initialized with the audio data.
+        """
+        return cls(input_audio=RawAudio.from_audio(audio))
+
+    def to_openai(self) -> Dict[str, Union[str, Dict[str, str]]]:
+        r"""Converts the chunk to the OpenAI format.
+
+        Returns:
+            A dictionary representing the audio chunk in the OpenAI format.
+        """
+        content = (
+            self.input_audio.data.decode("utf-8") if isinstance(self.input_audio.data, bytes) else self.input_audio.data
+        )
+        return {
+            "type": self.type,
+            "input_audio": RawAudio(data=content, format=self.input_audio.format).model_dump(),
+        }
+
+    @classmethod
+    def from_openai(cls, openai_chunk: Dict[str, Union[str, Dict[str, str]]]) -> "AudioChunk":
+        r"""Converts the OpenAI chunk to the Mistral format.
+
+        Args:
+            openai_chunk: A dictionary representing the audio chunk in the OpenAI format.
+
+        Returns:
+            An AudioChunk instance initialized with the data from the OpenAI chunk.
+        """
+        return cls.model_validate(openai_chunk)
+
+
 class TextChunk(BaseContentChunk):
     r"""Text chunk.
 
@@ -160,12 +364,40 @@ class TextChunk(BaseContentChunk):
         return self.model_dump()
 
     @classmethod
-    def from_openai(cls, messages: Dict[str, Union[str, Dict[str, str]]]) -> "TextChunk":
+    def from_openai(cls, openai_chunk: Dict[str, Union[str, Dict[str, str]]]) -> "TextChunk":
         r"""Converts the OpenAI chunk to the Mistral format."""
-        return cls.model_validate(messages)
+        return cls.model_validate(openai_chunk)
 
 
-ContentChunk = Annotated[Union[TextChunk, ImageChunk, ImageURLChunk], Field(discriminator="type")]
+class ThinkChunk(BaseContentChunk):
+    r"""Thinking chunk.
+
+    Attributes:
+        type: The type of the chunk, which is always ChunkTypes.thinking.
+        thinking: The list of text chunks of the thinking.
+        closed: Whether the thinking chunk is closed or not.
+    """
+
+    type: Literal[ChunkTypes.thinking] = ChunkTypes.thinking
+    thinking: str
+    closed: bool = Field(default=True, description="Whether the thinking chunk is closed or not.")
+
+    def to_openai(self) -> Dict[str, Union[str, Dict[str, str]]]:
+        r"""Converts the chunk to the OpenAI format."""
+        return self.model_dump()
+
+    @classmethod
+    def from_openai(cls, openai_chunk: Dict[str, Union[str, Dict[str, str]]]) -> "ThinkChunk":
+        r"""Converts the OpenAI chunk to the Mistral format."""
+        return cls.model_validate(openai_chunk)
+
+
+ContentChunk = Annotated[
+    Union[TextChunk, ImageChunk, ImageURLChunk, AudioChunk, AudioURLChunk, ThinkChunk], Field(discriminator="type")
+]
+UserContentChunk = Annotated[
+    Union[TextChunk, ImageChunk, ImageURLChunk, AudioChunk, AudioURLChunk], Field(discriminator="type")
+]
 
 
 def _convert_openai_content_chunks(openai_content_chunks: Dict[str, Union[str, Dict[str, str]]]) -> ContentChunk:
@@ -182,6 +414,12 @@ def _convert_openai_content_chunks(openai_content_chunks: Dict[str, Union[str, D
         return ImageURLChunk.from_openai(openai_content_chunks)
     elif content_type == ChunkTypes.image:
         return ImageChunk.from_openai(openai_content_chunks)
+    elif content_type == ChunkTypes.input_audio:
+        return AudioChunk.from_openai(openai_content_chunks)
+    elif content_type == ChunkTypes.audio_url:
+        return AudioURLChunk.from_openai(openai_content_chunks)
+    elif content_type == ChunkTypes.thinking:
+        return ThinkChunk.from_openai(openai_content_chunks)
     else:
         raise ValueError(f"Unknown content chunk type: {content_type}")
 
@@ -243,7 +481,7 @@ class UserMessage(BaseMessage):
     """
 
     role: Literal[Roles.user] = Roles.user
-    content: Union[str, List[ContentChunk]]
+    content: Union[str, List[UserContentChunk]]
 
     def to_openai(self) -> Dict[str, Union[str, List[Dict[str, Union[str, Dict[str, Any]]]]]]:
         r"""Converts the message to the OpenAI format."""
@@ -277,7 +515,7 @@ class SystemMessage(BaseMessage):
     """
 
     role: Literal[Roles.system] = Roles.system
-    content: str
+    content: Union[str, List[Union[TextChunk, ThinkChunk]]]
 
     def to_openai(self) -> Dict[str, Union[str, List[Dict[str, Union[str, Dict[str, Any]]]]]]:
         r"""Converts the message to the OpenAI format."""
@@ -305,7 +543,7 @@ class AssistantMessage(BaseMessage):
     """
 
     role: Literal[Roles.assistant] = Roles.assistant
-    content: Optional[str] = None
+    content: Optional[Union[str, List[Union[TextChunk, ThinkChunk]]]] = None
     tool_calls: Optional[List[ToolCall]] = None
     prefix: bool = False
 
@@ -314,8 +552,12 @@ class AssistantMessage(BaseMessage):
         out_dict: dict[str, Union[str, List[Dict[str, Union[str, Dict[str, Any]]]]]] = {
             "role": self.role,
         }
-        if self.content is not None:
+        if self.content is None:
+            pass
+        elif isinstance(self.content, str):
             out_dict["content"] = self.content
+        else:
+            out_dict["content"] = [chunk.to_openai() for chunk in self.content]
         if self.tool_calls is not None:
             out_dict["tool_calls"] = [tool_call.to_openai() for tool_call in self.tool_calls]
 
@@ -335,11 +577,19 @@ class AssistantMessage(BaseMessage):
             if openai_tool_calls is not None
             else None
         )
+        openai_content = openai_message.get("content", None)
+        content: Optional[Union[str, List[ContentChunk]]] = None
+        if openai_content is None or isinstance(openai_content, str):
+            content = openai_content
+        elif isinstance(openai_content, list):
+            content = [_convert_openai_content_chunks(chunk) for chunk in openai_content]
+        else:
+            raise ValueError(f"Unknown content type: {type(openai_content)}")
 
         return cls.model_validate(
             {
                 "role": openai_message["role"],
-                "content": openai_message.get("content"),
+                "content": content,
                 "tool_calls": tools_calls,
             }
         )
