@@ -25,16 +25,17 @@ from mistral_common.protocol.instruct.validator import (
     MistralRequestValidatorV5,
     ValidationMode,
 )
-from mistral_common.tokens.tokenizers.base import InstructRequest, TokenizerVersion
+from mistral_common.tokens.tokenizers.base import InstructRequest, InstructTokenizer, TokenizerVersion
 from mistral_common.tokens.tokenizers.image import ImageEncoder
 from mistral_common.tokens.tokenizers.instruct import InstructTokenizerV7
 from mistral_common.tokens.tokenizers.mistral import MistralTokenizer
 from mistral_common.tokens.tokenizers.tekken import Tekkenizer
 from tests.test_tekken import _quick_vocab
+from tests.test_tokenizer_v7_audio import _get_tekkenizer_with_audio
 
 
-@pytest.fixture(scope="session")
-def tekkenizer() -> InstructTokenizerV7:
+@pytest.fixture
+def no_audio_tekkenizer() -> InstructTokenizerV7:
     tokenizer = Tekkenizer(
         _quick_vocab([b"a", b"b", b"c", b"f", b"de"]),
         list(Tekkenizer.DEPRECATED_SPECIAL_TOKENS),
@@ -46,6 +47,11 @@ def tekkenizer() -> InstructTokenizerV7:
     return InstructTokenizerV7(tokenizer)
 
 
+@pytest.fixture
+def with_audio_tekkenizer() -> InstructTokenizerV7:
+    return _get_tekkenizer_with_audio()
+
+
 @pytest.fixture(scope="session")
 def spm_tokenizer() -> InstructTokenizerV7:
     tokenizer = MistralTokenizer.v7(is_mm=True).instruct_tokenizer
@@ -355,8 +361,13 @@ def test_encode_chat_completion() -> None:
         ),
     ],
 )
-def test_truncation(tekkenizer: InstructTokenizerV7, messages: List[ChatMessage], truncated_text: str) -> None:
-    tokenized = tekkenizer.encode_instruct(InstructRequest(messages=messages, truncate_at_max_tokens=15))
+@pytest.mark.parametrize("tekkenizer", ["no_audio_tekkenizer", "with_audio_tekkenizer"])
+def test_truncation(
+    request: pytest.FixtureRequest, tekkenizer: str, messages: List[ChatMessage], truncated_text: str
+) -> None:
+    tokenizer: InstructTokenizer = request.getfixturevalue(tekkenizer)
+
+    tokenized = tokenizer.encode_instruct(InstructRequest(messages=messages, truncate_at_max_tokens=15))
     assert tokenized.text == truncated_text, f"{tokenized.text} != {truncated_text}"
 
 
@@ -373,9 +384,11 @@ def test_truncation(tekkenizer: InstructTokenizerV7, messages: List[ChatMessage]
         ],
     ],
 )
-def test_truncation_failed(tekkenizer: InstructTokenizerV7, messages: List[ChatMessage]) -> None:
+@pytest.mark.parametrize("tekkenizer", ["no_audio_tekkenizer", "with_audio_tekkenizer"])
+def test_truncation_failed(request: pytest.FixtureRequest, tekkenizer: str, messages: List[ChatMessage]) -> None:
+    tokenizer = request.getfixturevalue(tekkenizer)
     with pytest.raises(TokenizerException):
-        tekkenizer.encode_instruct(InstructRequest(messages=messages, truncate_at_max_tokens=9))
+        tokenizer.encode_instruct(InstructRequest(messages=messages, truncate_at_max_tokens=9))
 
 
 def test_from_model() -> None:
@@ -417,8 +430,10 @@ def test_from_model() -> None:
         assert tokenizer.instruct_tokenizer.image_encoder is not None
 
 
-def test_assistant_tool_call_and_content(tekkenizer: InstructTokenizerV7) -> None:
-    request: InstructRequest = InstructRequest(
+@pytest.mark.parametrize("tekkenizer", ["no_audio_tekkenizer", "with_audio_tekkenizer"])
+def test_assistant_tool_call_and_content(request: pytest.FixtureRequest, tekkenizer: str) -> None:
+    tokenizer = request.getfixturevalue(tekkenizer)
+    instruct_request: InstructRequest = InstructRequest(
         available_tools=[
             Tool(function=Function(name="t1", parameters={})),
             Tool(function=Function(name="t2", parameters={})),
@@ -434,7 +449,7 @@ def test_assistant_tool_call_and_content(tekkenizer: InstructTokenizerV7) -> Non
             ),
         ],
     )
-    tokenized = tekkenizer.encode_instruct(request)
+    tokenized = tokenizer.encode_instruct(instruct_request)
     tokens = tokenized.tokens
     text = tokenized.text
 
@@ -448,14 +463,13 @@ def test_assistant_tool_call_and_content(tekkenizer: InstructTokenizerV7) -> Non
     )
 
     # make sure it also works end to end
-    tools = request.available_tools
-    chat_completion_request = ChatCompletionRequest(
-        **request.model_dump(exclude={"system_prompt", "truncate_at_max_tokens", "available_tools"}), tools=tools
-    )
+    tools = instruct_request.available_tools
+    exclude = {"system_prompt", "truncate_at_max_tokens", "available_tools"}
+    chat_completion_request = ChatCompletionRequest(**instruct_request.model_dump(exclude=exclude), tools=tools)
     validator = MistralRequestValidatorV5(mode=ValidationMode.finetuning)
     normalizer = InstructRequestNormalizerV7.normalizer()
 
-    mistral_tokenizer = MistralTokenizer(tekkenizer, validator, normalizer)
+    mistral_tokenizer = MistralTokenizer(tokenizer, validator, normalizer)
     tokens_2 = mistral_tokenizer.encode_chat_completion(chat_completion_request)
 
     assert tokens == tokens_2.tokens
